#!/usr/bin/env python

# Notes
# This plugin "presses" the button for garage door openers. A relay is connected to the raspPi's GPIO pins and the garage 
# door opener. The URLs "/gp1" or "/gp2" will briefly activate the relay and simulate the button press.
# Reed switches can also be connected to the RaspPi to check whether the door is open or closed.
# The URLs "/gs1" or "/gs2" will return the status of the reed switches

import web, json, time,sys
import gv # Get access to ospi settings
from urls import urls # Get access to ospi URLs
import RPi.GPIO as GPIO
from ospi import verifyLogin
from ospi import baseurl
from ospi import CPU_temperature

gv.plugin_menu.append(['Garage Doors', '/gi'])

# Setup urls for button presses and status page
urls.extend(['/gp1', 'plugins.garagecontrol.press_1', '/gp2', 'plugins.garagecontrol.press_2', '/gi', 'plugins.garagecontrol.ginfo'])
urls.extend(['/gs1', 'plugins.garagecontrol.get_status_1', '/gs2', 'plugins.garagecontrol.get_status_2'])

#### defines ####
use_door_sensor_1 = 1 # change to 0 if not using door sensor 1
use_door_sensor_2 = 1 # change to 0 if not using door sensor 2
door_motion_wait = 12 # seconds to wait for door to close
button_press_length = 1 # seconds to hold down the button
pin_door_button_1 = 12 # GPIO BOARD pin for door 1 relay
pin_door_button_2 = 16 # GPIO BOARD pin for door 2 relay
pin_door_sensor_1 = 18 # GPIO BOARD pin for door 1 reed switch
pin_door_sensor_2 = 22 # GPIO BOARD pin for door 2 reed switch
garage_name_1 = 'Big door' # descriptive name for door 1
garage_name_2 = 'Small door' # descriptive name for door 2
gpio_active = GPIO.LOW # GPIO state to activate relay
gpio_inactive = GPIO.HIGH # GPIO state when relay is inactivate
	
### setup GPIO ####
GPIO.setwarnings(False)
GPIO.setmode(GPIO.BOARD) # Note that we're using board numbering

# GPIO to relays setup
GPIO.setup(pin_door_button_1, GPIO.OUT, initial=gpio_inactive)
GPIO.setup(pin_door_button_2, GPIO.OUT, initial=gpio_inactive)
# Input with pullup to read garage door state, 0, Low, False = Closed
GPIO.setup(pin_door_sensor_1, GPIO.IN, pull_up_down=GPIO.PUD_UP)
GPIO.setup(pin_door_sensor_2, GPIO.IN, pull_up_down=GPIO.PUD_UP)

# Simulate a button push by turning a relay on then off
def press_button_1():
	verifyLogin()
	GPIO.output(pin_door_button_1, gpio_active)
	time.sleep(button_press_length)
	GPIO.output(pin_door_button_1, gpio_inactive)
	print('Press command sent to garage button 1 - '+ garage_name_1)

def press_button_2():
	verifyLogin()
	GPIO.output(pin_door_button_2, gpio_active)
	time.sleep(button_press_length)
	GPIO.output(pin_door_button_2, gpio_inactive)
	print('Press command sent to garage button 2 - '+ garage_name_2)
	
def get_door_status_1():
	verifyLogin()
	if use_door_sensor_1 == 1:
		if GPIO.input(pin_door_sensor_1):
			door_status_1 = 'open'
		else:
			door_status_1 = 'closed'
	else:
		door_status_1 = 'not enabled'
	print('Door sensor 1: ' + garage_name_1 + ' is ' + door_status_1)
	return(door_status_1)
	
def get_door_status_2():
	verifyLogin()
	if use_door_sensor_2 == 1:
		if GPIO.input(pin_door_sensor_2):
			door_status_2 = 'open'
		else:
			door_status_2 = 'closed'
	else:
		door_status_2 = 'not enabled'
	print('Door sensor 2: ' + garage_name_2 + ' is ' + door_status_2)
	return(door_status_2)

#### classes for urls ####

class press_1:
	def GET(self):
		press_button_1()
		return('Button 1 was pressed')
        # add code here to wait and check door status and refresh page?

class press_2:
	def GET(self):
		press_button_2()
		return('Button 2 was pressed')
		# add code here to wait and check door status and refresh page?

class get_status_1:
    def GET(self):
		return(get_door_status_1())
		

class get_status_2:
    def GET(self):
		return(get_door_status_2())
		
#Show a very basic garage status and control html page
#Must have garages.html in the templates folder
class ginfo:

    def GET(self):
        # get door status and send it to garages.html
		door_status_1 = get_door_status_1()
		door_status_2 = get_door_status_2()
		gv.baseurl = baseurl()
		gv.cputemp = CPU_temperature()
		render = web.template.render('templates', globals={ 'gv': gv, 'str': str, 'eval': eval })
		garagestatus=[door_status_1, door_status_2]            
		return render.garages(garagestatus)
